@@ -277,10 +277,13 @@ class XenHypervisor(BaseHypervisor):
     """Return information about the node.
 
     The return value is a dict, which has to have the following items:
-      (all values in MiB)
+      (memory values in MiB)
       - memory_total: the total memory size on the node
       - memory_free: the available memory on the node for instances
       - memory_dom0: the memory used by the node itself, if available
+      - nr_cpus: total number of CPUs
+      - nr_nodes: in a NUMA system, the number of domains
+      - nr_sockets: the number of physical CPU sockets in the node
 
     """
     # note: in xen 3, memory has changed to total_memory
@@ -291,6 +294,7 @@ class XenHypervisor(BaseHypervisor):
 
     xmoutput = result.stdout.splitlines()
     result = {}
+    cores_per_socket = threads_per_core = nr_cpus = None
     for line in xmoutput:
       splitfields = line.split(":", 1)
 
@@ -302,7 +306,18 @@ class XenHypervisor(BaseHypervisor):
         elif key == 'free_memory':
           result['memory_free'] = int(val)
         elif key == 'nr_cpus':
-          result['cpu_total'] = int(val)
+          nr_cpus = result['cpu_total'] = int(val)
+        elif key == 'nr_nodes':
+          result['cpu_nodes'] = int(val)
+        elif key == 'cores_per_socket':
+          cores_per_socket = int(val)
+        elif key == 'threads_per_core':
+          threads_per_core = int(val)
+
+    if (cores_per_socket is not None and
+        threads_per_core is not None and nr_cpus is not None):
+      result['cpu_sockets'] = nr_cpus / (cores_per_socket * threads_per_core)
+
     dom0_info = self.GetInstanceInfo("Domain-0")
     if dom0_info is not None:
       result['memory_dom0'] = dom0_info[2]
