@@ -28,6 +28,7 @@ import tempfile
 import os.path
 import os
 import md5
+import signal
 import socket
 import shutil
 import re
@@ -42,6 +43,9 @@ from ganeti.utils import IsProcessAlive, Lock, Unlock, RunCmd, \
      SetEtcHostsEntry, RemoveEtcHostsEntry
 from ganeti.errors import LockError, UnitParseError
 
+def _ChildHandler(signal, stack):
+  global _ChildFlag
+  _ChildFlag = True
 
 class GanetiTestCase(unittest.TestCase):
   def assertFileContent(self, file_name, content):
@@ -57,13 +61,10 @@ class GanetiTestCase(unittest.TestCase):
 
 class TestIsProcessAlive(unittest.TestCase):
   """Testing case for IsProcessAlive"""
+
   def setUp(self):
-    # create a zombie and a (hopefully) non-existing process id
-    self.pid_zombie = os.fork()
-    if self.pid_zombie == 0:
-      os._exit(0)
-    elif self.pid_zombie < 0:
-      raise SystemError("can't fork")
+    global _ChildFlag
+    # create a (most probably) non-existing process-id
     self.pid_non_existing = os.fork()
     if self.pid_non_existing == 0:
       os._exit(0)
@@ -71,7 +72,18 @@ class TestIsProcessAlive(unittest.TestCase):
       os.waitpid(self.pid_non_existing, 0)
     else:
       raise SystemError("can't fork")
+    _ChildFlag = False
+    # Use _ChildHandler for SIGCHLD
+    self.chldOrig = signal.signal(signal.SIGCHLD, _ChildHandler)
+    # create a zombie
+    self.pid_zombie = os.fork()
+    if self.pid_zombie == 0:
+      os._exit(0)
+    elif self.pid_zombie < 0:
+      raise SystemError("can't fork")
 
+  def tearDown(self):
+    signal.signal(signal.SIGCHLD, self.chldOrig)
 
   def testExists(self):
     mypid = os.getpid()
@@ -79,10 +91,20 @@ class TestIsProcessAlive(unittest.TestCase):
                  "can't find myself running")
 
   def testZombie(self):
+    global _ChildFlag
+    timeout = 10
+
+    while not _ChildFlag:
+      if timeout >= 0:
+        time.sleep(0.2)
+        timeout -= 0.2
+      else:
+        self.fail("timed out waiting for child's signal")
+        break # not executed...
+
     self.assert_(not IsProcessAlive(self.pid_zombie),
                  "zombie not detected as zombie")
 
-
   def testNotExisting(self):
     self.assert_(not IsProcessAlive(self.pid_non_existing),
                  "noexisting process detected")
@@ -166,8 +188,8 @@ class TestRunCmd(unittest.TestCase):
     self.assertEqual(result.output, "A" + self.magic + "B" + self.magic)
 
   def testSignal(self):
-    """Test standard error"""
-    result = RunCmd("/bin/sh -c 'kill -15 $$'")
+    """Test signal"""
+    result = RunCmd(["python", "-c", "import os; os.kill(os.getpid(), 15)"])
     self.assertEqual(result.signal, 15)
 
   def testListRun(self):
@@ -544,12 +566,20 @@ class TestTcpPing(unittest.TestCase):
 
   def testTcpPingToLocalHostAccept(self):
     self.assert_(TcpPing(constants.LOCALHOST_IP_ADDRESS,
-                         constants.LOCALHOST_IP_ADDRESS,
                          self.listenerport,
                          timeout=10,
-                         live_port_needed=True),
+                         live_port_needed=True,
+                         source=constants.LOCALHOST_IP_ADDRESS,
+                         ),
                  "failed to connect to test listener")
 
+    self.assert_(TcpPing(constants.LOCALHOST_IP_ADDRESS,
+                         self.listenerport,
+                         timeout=10,
+                         live_port_needed=True,
+                         ),
+                 "failed to connect to test listener (no source)")
+
 
 class TestTcpPingDeaf(unittest.TestCase):
   """Testcase for TCP version of ping - against non listen(2)ing port"""
@@ -565,20 +595,36 @@ class TestTcpPingDeaf(unittest.TestCase):
 
   def testTcpPingToLocalHostAcceptDeaf(self):
     self.failIf(TcpPing(constants.LOCALHOST_IP_ADDRESS,
-                        constants.LOCALHOST_IP_ADDRESS,
                         self.deaflistenerport,
                         timeout=constants.TCP_PING_TIMEOUT,
-                        live_port_needed=True), # need successful connect(2)
+                        live_port_needed=True,
+                        source=constants.LOCALHOST_IP_ADDRESS,
+                        ), # need successful connect(2)
                 "successfully connected to deaf listener")
 
+    self.failIf(TcpPing(constants.LOCALHOST_IP_ADDRESS,
+                        self.deaflistenerport,
+                        timeout=constants.TCP_PING_TIMEOUT,
+                        live_port_needed=True,
+                        ), # need successful connect(2)
+                "successfully connected to deaf listener (no source addr)")
+
   def testTcpPingToLocalHostNoAccept(self):
     self.assert_(TcpPing(constants.LOCALHOST_IP_ADDRESS,
-                         constants.LOCALHOST_IP_ADDRESS,
                          self.deaflistenerport,
                          timeout=constants.TCP_PING_TIMEOUT,
-                         live_port_needed=False), # ECONNREFUSED is OK
+                         live_port_needed=False,
+                         source=constants.LOCALHOST_IP_ADDRESS,
+                         ), # ECONNREFUSED is OK
                  "failed to ping alive host on deaf port")
 
+    self.assert_(TcpPing(constants.LOCALHOST_IP_ADDRESS,
+                         self.deaflistenerport,
+                         timeout=constants.TCP_PING_TIMEOUT,
+                         live_port_needed=False,
+                         ), # ECONNREFUSED is OK
+                 "failed to ping alive host on deaf port (no source addr)")
+
 
 class TestListVisibleFiles(unittest.TestCase):
   """Test case for ListVisibleFiles"""
